#![allow(dead_code)]

mod tag;
mod util;
mod runtime;
mod interface; pub use interface::*;

fn assert<T>(expected:T, actual:T) where T:std::fmt::Display, T:std::cmp::PartialEq
{
    print!(" ({}) ", if expected == actual { " " } else { "!" });
    println!("'{}' = '{}'", expected, actual);
}

pub fn test()
{
    {
        println!("Boolean");
        let mut dat = Boolean::new();
        assert::<bool>(false, dat.get());
        dat.set(true);
        assert::<bool>(true, dat.get());
        dat.set(false);
        assert::<bool>(false, dat.get());
    }

    {
        println!("Natural");
        let mut dat = Natural::new();
        assert::<u64>(0, dat.get());
        dat.set(50);
        assert::<u64>(50, dat.get());
        dat.set(25000);
        assert::<u64>(25000, dat.get());
    }

    {
        println!("Integer");
        let mut dat = Integer::new();
        assert::<i64>(0, dat.get());
        dat.set(-12);
        assert::<i64>(-12, dat.get());
        dat.set(38695);
        assert::<i64>(38695, dat.get());
    }

    {
        println!("Block");
        let mut dat = Block::new(4);
        dat.set(vec![1, 2, 3, 4]);
        assert::<usize>(4, dat.get().len());
        assert::<u8>(1, dat.get()[0]);
        assert::<u8>(2, dat.get()[1]);
        assert::<u8>(3, dat.get()[2]);
        assert::<u8>(4, dat.get()[3]);
    }

    {
        println!("Sequence");
        let mut dat = Sequence::new();
        assert::<String>(String::from(""), dat.get());
        dat.set("hello");
        assert::<String>(String::from("hello"), dat.get());
    }

    {
        println!("Array<Natural>");
        let dat = Array::new(4, natural());
        Natural::from(dat.at(0)).unwrap().set(250);
        Natural::from(dat.at(1)).unwrap().set(500);
        Natural::from(dat.at(2)).unwrap().set(750);
        Natural::from(dat.at(3)).unwrap().set(1000);
        assert::<usize>(4, dat.length());
        assert::<u64>(250, Natural::from(dat.at(0)).unwrap().get());
        assert::<u64>(500, Natural::from(dat.at(1)).unwrap().get());
        assert::<u64>(750, Natural::from(dat.at(2)).unwrap().get());
        assert::<u64>(1000,Natural::from(dat.at(3)).unwrap().get());
    }

    /* {
        println!("List - Undefined");

        println!("List - Variable");
        let mut dat = List::new(varying());
        dat.insert(usize::MAX, *Integer::with(15));
        match Type::from(dat.at(0)) {
            Type::Integer(value) => {
                println!(" = {}", value.get());
            }
            _ => { println!(" - Not Integer"); }
        }

        println!("List<Sequence>");
        let mut dat = List::new(sequence());
        dat.insert(usize::MAX, *Sequence::with("hello"));
        dat.insert(usize::MAX, *Sequence::with("world"));
        dat.insert(usize::MAX, *Sequence::with("konnichiwa"));
        dat.insert(usize::MAX, *Sequence::with("konbanwa"));
        dat.insert(usize::MAX, *Sequence::with("tasogare"));
        println!(" = {}", Integer::from(dat.at(0)).unwrap().get());
    } */

    /*{
        println!("Sparse");

    }*/

    /*{
        println!("Schema");

    }*/

    /*{
        println!("Record");

    }*/
}
